Package: tidyusmacro
Title: Downloading and Cleaning U.S. Macroeconomic Data
Version: 0.2.0
Authors@R: 
    person("Mike", "Konczal", email = "konczal@gmail.com", role = c("aut", "cre"))
Description: Utilities to retrieve and tidy U.S. macroeconomic data series
    from public government data providers. Functions streamline access to series
    from the Federal Reserve Bank of St. Louis Federal Reserve Economic Data (FRED),
    the Bureau of Labor Statistics flat files, and the Bureau of Economic
    Analysis National Income and Product Accounts tables, then return
    consistent, tidy data frames ready for modeling and graphics. The package includes
    helpers for date alignment, log-linear projections, and common macro diagnostics,
    along with convenience plot builders for quick publication-quality charts.
License: MIT + file LICENSE
URL: https://github.com/mtkonczal/tidyusmacro,
        https://www.mikekonczal.com/tidyusmacro/
BugReports: https://github.com/mtkonczal/tidyusmacro/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
LazyData: true
Imports: dplyr, ggplot2, httr, magrittr, purrr, readr, rlang, stringi,
        tidyr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-10 21:33:37 UTC; mtkonczal
Author: Mike Konczal [aut, cre]
Maintainer: Mike Konczal <konczal@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 07:00:02 UTC
Built: R 4.4.3; ; 2026-06-12 07:45:04 UTC; unix

Package: tidyusmacro
Title: Downloading and Cleaning U.S. Macroeconomic Data
Version: 0.2.0
Authors@R: 
    person("Mike", "Konczal", email = "konczal@gmail.com", role = c("aut", "cre"))
Description: Utilities to retrieve and tidy U.S. macroeconomic data series
    from public government data providers. Functions streamline access to series
    from the Federal Reserve Bank of St. Louis Federal Reserve Economic Data (FRED),
    the Bureau of Labor Statistics flat files, and the Bureau of Economic
    Analysis National Income and Product Accounts tables, then return
    consistent, tidy data frames ready for modeling and graphics. The package includes
    helpers for date alignment, log-linear projections, and common macro diagnostics,
    along with convenience plot builders for quick publication-quality charts.
License: MIT + file LICENSE
URL: https://github.com/mtkonczal/tidyusmacro,
        https://www.mikekonczal.com/tidyusmacro/
BugReports: https://github.com/mtkonczal/tidyusmacro/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
LazyData: true
Imports: dplyr, ggplot2, httr, magrittr, purrr, readr, rlang, stringi,
        tidyr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-10 21:33:37 UTC; mtkonczal
Author: Mike Konczal [aut, cre]
Maintainer: Mike Konczal <konczal@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 07:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(date_breaks_gg)
export(date_breaks_n)
export(esp_navy)
export(esp_pal)
export(getBLSFiles)
export(getDallasTrimPCE)
export(getFRED)
export(getNIPAFiles)
export(getPCEInflation)
export(getUnrateFRED)
export(logLinearProjection)
export(scale_color_esp)
export(scale_colour_esp)
export(scale_fill_esp)
export(theme_esp)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,case_when)
importFrom(dplyr,coalesce)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,lag)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(grid,unit)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(httr,status_code)
importFrom(httr,user_agent)
importFrom(magrittr,"%>%")
importFrom(purrr,compact)
importFrom(purrr,pmap)
importFrom(purrr,reduce)
importFrom(readr,cols)
importFrom(readr,read_csv)
importFrom(readr,read_tsv)
importFrom(rlang,.data)
importFrom(stats,lm)
importFrom(stats,predict)
importFrom(utils,data)
importFrom(utils,globalVariables)
importFrom(utils,head)

# tidyusmacro

Utilities to retrieve and tidy U.S. macroeconomic data series from public government data providers. Functions streamline access to series from the Federal Reserve Bank of St. Louis Federal Reserve Economic Data (FRED), the Bureau of Labor Statistics (BLS) flat files, and the Bureau of Economic Analysis (BEA) National Income and Product Accounts (NIPA) tables, then return consistent, tidy data frames ready for modeling and graphics.

The package includes helpers for date alignment, log-linear projections, and common macro diagnostics, along with convenience plot builders for quick publication-quality charts.

## Installation

```r
# Install from CRAN
install.packages("tidyusmacro")

# Or the development version from GitHub
devtools::install_github("mtkonczal/tidyusmacro")
```

## Functions

### Data Retrieval

#### `getFRED`
Downloads and merges economic data series from the Federal Reserve Economic Data (FRED) API.

```r
# Named arguments give friendly column names
fred_data <- getFRED(prime_epop = "LNS12300060", cpi = "CPIAUCSL")

# Unnamed arguments use lowercase ticker as column name
fred_data <- getFRED("UNRATE", "PAYEMS")
```

#### `getBLSFiles`
Downloads and processes data from Bureau of Labor Statistics flat files. Supports CPI, ECI, JOLTS, CPS, CES, and CEX (Consumer Expenditure Survey).

```r
cpi_data <- getBLSFiles(data_source = "cpi", email = "user@example.com")
jolts_data <- getBLSFiles(data_source = "jolts", email = "user@example.com")
```

#### `getNIPAFiles`
Downloads and formats BEA NIPA data flat files, either monthly or quarterly values.

```r
nipa_quarterly <- getNIPAFiles(type = "Q")
nipa_monthly <- getNIPAFiles(type = "M")
```

#### `getPCEInflation`
Loads and processes Personal Consumption Expenditures (PCE) inflation data with weights and growth measures.

```r
pce_monthly <- getPCEInflation("M")
pce_quarterly <- getPCEInflation("Q")
```

#### `getUnrateFRED`
Convenience function to download unemployment level and labor force from FRED and calculate the unemployment rate.

```r
unrate_data <- getUnrateFRED()
```

#### `getDallasTrimPCE`
Builds the component-level panel underlying the Dallas Fed Trimmed Mean PCE inflation rate: monthly price changes, Fisher expenditure-share weights, and flags for which components are trimmed each month. Useful for replicating the trimmed-mean rate or analyzing what gets trimmed.

```r
# Default 24/31 Dallas Fed trim
panel <- getDallasTrimPCE()

# Replicate the monthly trimmed-mean rate
panel |>
  dplyr::filter(!is_trimmed) |>
  dplyr::group_by(date) |>
  dplyr::summarize(trim_pce = weighted.mean(price_change, weight))
```

### Statistical Functions

#### `logLinearProjection`
Performs log-linear projections on historical data. Designed for use within dplyr verbs.

```r
library(dplyr)

data %>%
  mutate(projection = logLinearProjection(
    date = date,
    value = gdp,
    start_date = "2015-01-01",
    end_date = "2019-12-01"
  ))
```

### Visualization

#### `theme_esp`
Custom ggplot2 theme for Economic Security Project graphics with cream background and clean styling.

```r
library(ggplot2)

ggplot(data, aes(date, value)) +
  geom_line(color = esp_navy) +
  theme_esp()
```

#### `scale_color_esp` / `scale_fill_esp`
ESP-branded color scales for ggplot2.

```r
ggplot(data, aes(date, value, color = category)) +
  geom_line() +
  scale_color_esp() +
  theme_esp()
```

#### `date_breaks_gg`
Creates intelligent date breaks for ggplot2 that always include the last data point.

```r
ggplot(data, aes(date, value)) +
  geom_line() +
  scale_x_date(breaks = date_breaks_gg(n = 6, last = max(data$date)))
```

#### `date_breaks_n`
Generates evenly spaced date breaks by selecting every nth unique date.

```r
ggplot(data, aes(date, value)) +
  geom_line() +
  scale_x_date(breaks = date_breaks_n(data$date, n = 6))
```

## Included Data

#### `cesDiffusionIndex`
A tibble with 250 rows mapping CES industry codes to industry titles.

```r
data(cesDiffusionIndex)
```

#### `dallasTrimPCEcomponents`
The 177-component dictionary used by `getDallasTrimPCE`, mapping Dallas Fed trimmed-mean PCE components to BEA NIPA series codes and line numbers (Table 2.4.4U).

```r
data(dallasTrimPCEcomponents)
```

## Dependencies

- dplyr
- ggplot2
- httr
- tidyr
- readr
- purrr
- rlang
- stringi
- magrittr

## License

This library is distributed under the MIT License.

#' Get Full Unemployment Rate from FRED
#'
#' Downloads the civilian unemployment level and labor force level from
#' FRED, and calculates the unemployment rate as
#' \eqn{\text{unemploy\_level} / \text{lf\_level}}.
#'
#' @return A tibble with columns:
#'   \item{date}{Observation date}
#'   \item{unemploy_level}{Civilian unemployment level (in thousands)}
#'   \item{lf_level}{Civilian labor force level (in thousands)}
#'   \item{full_unrate}{Unemployment rate (decimal)}
#'
#' @examples
#' \dontrun{
#'   getUnrateFRED()
#' }
#' @export
getUnrateFRED <- function() {
  getFRED(
    unemploy_level = "UNEMPLOY",
    lf_level       = "CLF16OV"
  ) %>%
    dplyr::mutate(full_unrate = .data$unemploy_level / .data$lf_level)
}

#' cesDiffusionIndex Dataset
#'
#' A tibble with 250 rows and 2 columns representing industry codes and corresponding industry titles.
#'
#' This dataset contains information on different industries, where each row corresponds to an industry
#' defined by its unique code and a descriptive title. It is useful for analyses that require linking industry
#' classifications to descriptive labels.
#'
#' @format A tibble with 250 rows and 2 variables:
#' \describe{
#'   \item{ces_industry_code}{A character vector containing the industry codes (e.g., "10-11330000").}
#'   \item{ces_industry_title}{A character vector containing the titles of the industries (e.g., "Logging").}
#' }
#'
#' @source U.S. Bureau of Labor Statistics (BLS)
#'
#' @examples
#' # Load the dataset
#' data(cesDiffusionIndex)
#'
"cesDiffusionIndex"

#' Create evenly spaced breaks
#'
#' @description Generate a sequence of date breaks for ggplot scales,
#'              taking every `n`th unique date.
#'
#' @param dates A vector of dates.
#' @param n Integer, keep every n-th date (default = 6).
#' @param decreasing Logical, if TRUE (default) sorts dates in descending order.
#'
#' @return A vector of dates suitable for use as ggplot2 axis breaks.
#' @export
#'
#' @examples
#' library(ggplot2)
#' library(dplyr)
#'
#' df <- tibble(
#'   date = seq.Date(as.Date("2020-01-01"), as.Date("2025-01-01"), by = "month"),
#'   value = rnorm(61)
#' )
#'
#' ggplot(df, aes(date, value)) +
#'   geom_line() +
#'   scale_x_date(breaks = date_breaks_n(df$date, 6))
date_breaks_n <- function(dates, n = 6, decreasing = TRUE) {
  dates <- sort(unique(dates), decreasing = decreasing)
  dates[seq_along(dates) %% n == 1]
}

#' Download and Process Bureau of Labor Statistics Data
#'
#' Downloads and processes data from Bureau of Labor Statistics (BLS) flat files.
#' Supports multiple data sources including CPI, ECI, JOLTS, CPS, CES, and others.
#' The function retrieves the main data file along with associated metadata files,
#' merges them, and returns a tidy tibble ready for analysis.
#'
#' @param data_source Character string specifying the BLS data source. Available options:
#'   \describe{
#'     \item{\code{"cpi"}}{Consumer Price Index - current data}
#'     \item{\code{"eci"}}{Employment Cost Index (quarterly)}
#'     \item{\code{"cex"}}{Consumer Expenditure Survey}
#'     \item{\code{"jolts"}}{Job Openings and Labor Turnover Survey}
#'     \item{\code{"cps"}}{Current Population Survey}
#'     \item{\code{"ces"}}{Current Employment Statistics - all series}
#'     \item{\code{"ces_allemp"}}{Current Employment Statistics - all employees, seasonally adjusted}
#'     \item{\code{"ces_total"}}{Current Employment Statistics - total nonfarm employment}
#'     \item{\code{"averageprice"}}{Average price data - current}
#'     \item{\code{"food"}}{Average price data - food items}
#'     \item{\code{"se"}}{State and metro area employment}
#'     \item{\code{"su"}}{State and local area unemployment}
#'   }
#' @param email Character string with your email address. Required by BLS for
#'   identifying API users. Set as the HTTP User-Agent header.
#'
#' @return A tibble containing the merged data with columns for:
#'   \item{series_id}{Unique identifier for each data series}
#'   \item{date}{Observation date}
#'   \item{value}{Numeric data value}
#'   \item{...}{Additional metadata columns vary by data source (e.g., item codes,
#'     industry codes, area codes)}
#'
#' @details
#' The function constructs URLs to BLS flat files at
#' \url{https://download.bls.gov/pub/time.series/}, downloads the series
#' metadata and auxiliary lookup tables, then downloads and merges the main
#' data file. Date parsing handles both monthly (most sources) and quarterly
#' (ECI) data frequencies.
#'
#' @examples
#' \dontrun{
#'   # Download CPI data
#'   cpi_data <- getBLSFiles("cpi", "your.email@example.com")
#'
#'   # Download JOLTS data
#'   jolts_data <- getBLSFiles("jolts", "your.email@example.com")
#' }
#'
#' @importFrom readr read_tsv cols
#' @importFrom dplyr mutate left_join as_tibble case_when
#' @importFrom magrittr %>%
#' @export
getBLSFiles <- function(data_source, email) {
  # Validate the data source input
  available_sources <- c(
    "cpi",
    "eci",
    "jolts",
    "cps",
    "ces",
    "cex",
    "averageprice",
    "food",
    "ces_allemp",
    "ces_total",
    "se",
    "su"
  )
  if (!tolower(data_source) %in% available_sources) {
    stop(
      "Invalid data source. Choose one of: ",
      paste(available_sources, collapse = ", ")
    )
  }

  # fmt: skip
  file_mappings <- list(
    cpi          = c("cu", "data.0.Current", "series", "item", "area"),
    eci          = c("ci", "data.1.AllData", "series", "industry", "owner", "subcell", "occupation", "periodicity", "estimate"),
    cex          = c("cx", "data.1.AllData", "series", "category", "characteristics", "demographics", "item", "process"),
    jolts        = c("jt", "data.1.AllItems", "series", "industry", "state", "dataelement", "sizeclass"),
    cps          = c("ln", "data.1.AllData", "series", "ages", "occupation", "race", "sexs", "born", "lfst", "education"),
    ces          = c("ce", "data.0.AllCESSeries", "series", "datatype", "supersector", "industry"),
    ces_allemp   = c("ce", "data.01a.CurrentSeasAE", "series", "datatype", "supersector", "industry"),
    ces_total    = c("ce", "data.00a.TotalNonfarm.Employment", "series", "datatype", "supersector", "industry"),
    averageprice = c("ap", "data.0.Current", "series", "area", "item"),
    food         = c("ap", "data.3.Food", "series", "area", "item"),
    se           = c("sm", "data.0.Current", "series", "industry", "data_type", "supersector", "state", "area"),
    su           = c("la", "data.1.CurrentS", "series", "state_region_divison", "measure", "area", "area_type")
  )

  # 2. Extract the files based on the data_source variable
  files <- file_mappings[[data_source]]

  # Optional: error handling if the source doesn't exist
  if (is.null(files)) {
    stop("Unknown data source specified.")
  }

  # Construct the base URL: files are stored at
  # "https://download.bls.gov/pub/time.series/{folder}/{folder}.{file}"
  base_url <- paste0(
    "https://download.bls.gov/pub/time.series/",
    files[1],
    "/",
    files[1],
    "."
  )

  # Set HTTP user agent using the supplied email (BLS requires a contact
  # email in the user agent); restore the user's setting on exit per CRAN policy
  old_opts <- options(HTTPUserAgent = email)
  on.exit(options(old_opts), add = TRUE)

  # Define auxiliary files: the first two elements are reserved for URL construction
  # and the main data file, so auxiliary files start with the third element.
  aux_files <- files[-c(1, 2)]

  # Download and process the "series" file first.
  if ("series" %in% aux_files) {
    message("Downloading series file...")
    series_df <- readr::read_tsv(
      paste0(base_url, "series"),
      col_types = readr::cols()
    )
    # Clean series_id by removing spaces
    series_df <- dplyr::mutate(series_df, series_id = gsub(" ", "", series_id))
    # Remove "series" from the auxiliary file list as it is already processed.
    aux_files <- setdiff(aux_files, "series")
  } else {
    stop(
      "The 'series' file is required but was not found in the auxiliary file list."
    )
  }

  # BLS metadata columns that appear in most lookup files. These are useful for

  # understanding hierarchy (display_level) but have identical names across files.
  # We rename them with the file prefix to avoid .x/.y suffix collisions.
  # - display_level: hierarchy depth (0 = top level, higher = more detailed)
  # - selectable: whether item can be selected in BLS Data Finder UI
  # - sort_sequence: display order in BLS tools
  metadata_cols_to_rename <- c("display_level", "selectable", "sort_sequence")

  # Download and merge the remaining auxiliary files with series_df.
  for (file in aux_files) {
    message("Downloading file: ", file)
    tmp_df <- readr::read_tsv(paste0(base_url, file), col_types = readr::cols())

    # Rename metadata columns with file prefix to avoid collisions
    # e.g., "display_level" becomes "item_display_level" for the item file
    cols_to_rename <- intersect(names(tmp_df), metadata_cols_to_rename)
    if (length(cols_to_rename) > 0) {
      new_names <- paste0(file, "_", cols_to_rename)
      names(tmp_df)[match(cols_to_rename, names(tmp_df))] <- new_names
    }

    # Determine the join key(s) for this auxiliary file.
    # Most files use a simple "{file}_code" pattern, but some have compound keys
    # or non-standard naming conventions.
    join_key <- if (file == "datatype") {
      # CES datatype file uses "data_type_code" not "datatype_code"
      "data_type_code"
    } else if (data_source == "cex" && file == "characteristics") {
      # CEX characteristics: same characteristic code can mean different things

      # under different demographics (e.g., "01" = "All" for multiple groupings)
      c("demographics_code", "characteristics_code")
    } else if (data_source == "cex" && file == "item") {
      # CEX item: item_code is nested within subcategory_code
      c("subcategory_code", "item_code")
    } else {
      paste0(file, "_code")
    }

    # Handle join_key being length 1 OR length > 1
    missing_in_series <- setdiff(join_key, names(series_df))
    missing_in_tmp <- setdiff(join_key, names(tmp_df))

    if (length(missing_in_series) > 0 || length(missing_in_tmp) > 0) {
      warning(
        "Join key(s) missing. ",
        if (length(missing_in_series) > 0) {
          paste0(
            "Missing in series_df: ",
            paste(missing_in_series, collapse = ", "),
            ". "
          )
        } else {
          ""
        },
        if (length(missing_in_tmp) > 0) {
          paste0(
            "Missing in ",
            file,
            ": ",
            paste(missing_in_tmp, collapse = ", "),
            ". "
          )
        } else {
          ""
        },
        "Skipping file: ",
        file
      )
    } else {
      series_df <- dplyr::left_join(series_df, tmp_df, by = join_key)
    }
  }

  # Download the main data file.
  message("Downloading main data file: ", files[2])
  main_df <- readr::read_tsv(
    paste0(base_url, files[2]),
    col_types = readr::cols()
  )

  # Clean and convert the main data.
  main_df <- main_df %>%
    dplyr::mutate(
      series_id = gsub(" ", "", series_id),
      value = as.numeric(value)
    )

  # ECI is the only value that is base quarterly.
  if (data_source != "eci") {
    main_df$date <- as.Date(
      paste(substr(main_df$period, 2, 3), "01", main_df$year, sep = "/"),
      "%m/%d/%Y"
    )
  } else {
    quarterly_month <- dplyr::case_when(
      main_df$period == "Q01" ~ 3,
      main_df$period == "Q02" ~ 6,
      main_df$period == "Q03" ~ 9,
      main_df$period == "Q04" ~ 12,
      TRUE ~ NA_real_
    )
    main_df$date <- as.Date(
      paste(quarterly_month, "01", main_df$year, sep = "/"),
      "%m/%d/%Y"
    )
  }

  # Merge the main data with the series metadata.
  # First, identify columns that exist in both dataframes (except series_id).
  # Keep the main_df version of these columns (e.g., footnote_codes for the
  # observation is more relevant than footnote_codes for the series definition).
  message("Merging main data with series metadata...")

  common_cols <- setdiff(intersect(names(main_df), names(series_df)), "series_id")
  if (length(common_cols) > 0) {
    series_df <- series_df[, !names(series_df) %in% common_cols, drop = FALSE]
  }

  result_df <- dplyr::left_join(main_df, series_df, by = "series_id")

  # Return the final result as a tibble.
  dplyr::as_tibble(result_df)
}

#' ESP Theme and Color Scales
#'
#' Custom theme and color palette for Economic Security Project graphics.
#'
#' @param base_family Base font family for the theme. Defaults to "Public Sans".
#' @param ... Passed to the underlying ggplot2 scale functions.
#'
#' @return A ggplot2 theme or scale object.
#'
#' @name esp_theme
#' @importFrom ggplot2 theme_minimal theme element_rect element_blank element_line element_text scale_color_manual scale_fill_manual
#' @importFrom grid unit
#' @export
theme_esp <- function(base_family = "Public Sans") {
  ggplot2::theme_minimal(base_family = base_family) +
    ggplot2::theme(
      # Background
      plot.background = ggplot2::element_rect(fill = "#f4f2e4", color = NA),
      panel.background = ggplot2::element_rect(fill = "#f4f2e4", color = NA),

      # Remove gridlines
      panel.grid.minor = ggplot2::element_blank(),
      panel.grid.major.y = ggplot2::element_line(color = "grey80"),
      panel.grid.major.x = ggplot2::element_line(color = "grey80"),

      # Axis lines and ticks
      axis.line = ggplot2::element_line(color = "black", linewidth = 0.5),
      axis.ticks = ggplot2::element_line(color = "black", linewidth = 0.5),
      axis.ticks.length = grid::unit(5, "pt"),

      # Titles and text
      plot.title = ggplot2::element_text(
        size = 25,
        face = "bold",
        family = base_family,
        color = "black",
        margin = ggplot2::margin(b = 10, t = 10)
      ),
      plot.subtitle = ggplot2::element_text(
        size = 15,
        family = base_family,
        color = "black"
      ),
      plot.caption = ggplot2::element_text(
        size = 10,
        face = "italic",
        family = base_family,
        color = "black"
      ),
      axis.text = ggplot2::element_text(
        size = 12,
        face = "bold",
        family = base_family,
        color = "black"
      ),
      strip.text = ggplot2::element_text(
        face = "bold",
        size = 10,
        hjust = 0.5,
        family = base_family,
        color = "black"
      ),

      # Remove axis titles and legend
      axis.title.x = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_blank(),
      legend.position = "none",
      legend.title = ggplot2::element_blank(),
      legend.text = ggplot2::element_text(
        size = 12,
        family = base_family,
        color = "black"
      ),
      # Title alignment
      plot.title.position = "plot"
    )
}

#' ESP Color Palette
#'
#' Named vector of ESP-branded colors.
#' @export
esp_pal <- c(
  "Warm Navy" = "#2c3254",
  "Warm Red" = "#ff8361",
  "Soft Green" = "#70ad8f"
)

#' ESP Primary Color (Navy)
#'
#' A standalone color value for quick use.
#' @export
esp_navy <- esp_pal["Warm Navy"]

#' ESP Discrete Color Scale
#' @rdname esp_theme
#' @export
scale_color_esp <- function(...) {
  ggplot2::scale_color_manual(values = esp_pal, ...)
}

#' ESP Discrete Fill Scale
#' @rdname esp_theme
#' @export
scale_fill_esp <- function(...) {
  ggplot2::scale_fill_manual(values = esp_pal, ...)
}

#' Alias for American/British spelling
#' @rdname esp_theme
#' @export
scale_colour_esp <- scale_color_esp

#' Log-Linear Projection (data-masked, dplyr-native)
#'
#' Fits a log-linear trend \code{log(value) ~ t} on a calibration window and
#' projects it for rows on/after \code{start_date}. Designed for use inside
#' dplyr verbs (no need to pass \code{.}).
#'
#' @param date Bare column name for the date variable (coercible to Date).
#' @param value Bare column name for the positive numeric series to project.
#' @param start_date Date or string coercible to Date; start of calibration.
#' @param end_date Date or string coercible to Date; end of calibration.
#' @param group Optional bare column name to group by before projecting.
#' @param data Optional data frame. If omitted, uses the columns visible in
#'   the current data mask (e.g., inside \code{mutate()}) via
#'   \code{dplyr::pick()}.
#'
#' @return A numeric vector \code{projection} aligned to the input rows; \code{NA}
#'   before \code{start_date}. Respects grouping if \code{group} is supplied.
#' @export
#' @importFrom stats lm predict
logLinearProjection <- function(date, value, start_date, end_date, group = NULL, data = NULL) {
  rlang::check_installed(c("dplyr", "rlang"))

  date_q  <- rlang::enquo(date)
  value_q <- rlang::enquo(value)
  group_q <- rlang::enquo(group)

  # Catch calls using the pre-0.2.0 interface, which passed a data frame
  # first: logLinearProjection(tbl, "date_col", "value_col", ...)
  first_arg <- tryCatch(rlang::eval_tidy(date_q), error = function(e) NULL)
  if (is.data.frame(first_arg)) {
    stop(
      "As of tidyusmacro 0.2.0, logLinearProjection() takes bare column ",
      "names and is designed for use inside mutate(): ",
      "mutate(proj = logLinearProjection(date, value, start_date, end_date)). ",
      "The old interface logLinearProjection(tbl, \"date_col\", \"value_col\", ...) ",
      "is no longer supported. See ?logLinearProjection.",
      call. = FALSE
    )
  }

  if (is.null(data)) {
    # pick() replaces the deprecated cur_data_all(), but unlike
    # cur_data_all() it excludes grouping variables in a grouped mutate;
    # bind them back so `group` can still reference a grouping column.
    data <- dplyr::pick(dplyr::everything())
    grp_vals <- dplyr::cur_group()
    if (ncol(grp_vals) > 0) {
      data <- dplyr::bind_cols(
        grp_vals[rep(1, nrow(data)), , drop = FALSE],
        data
      )
    }
  }

  start_date <- as.Date(start_date)
  end_date   <- as.Date(end_date)

  has_group <- !rlang::quo_is_null(group_q)

  df <- data |>
    dplyr::mutate(
      .row_id = dplyr::row_number(),
      .date   = as.Date(!!date_q),
      .value  = suppressWarnings(as.numeric(!!value_q))
    )

  if (has_group) {
    df <- dplyr::group_by(df, !!group_q, .add = FALSE)
  }

  out <- df |>
    # sort by date within each group (or just by date if ungrouped)
    dplyr::arrange(.date, .by_group = TRUE) |>
    dplyr::mutate(
      .calib = .date >= start_date & .date <= end_date,
      .t     = as.numeric(.date)  # days since epoch
    ) |>
    dplyr::group_modify(function(d, ...) {
      fit_data <- d[d$.calib & is.finite(d$.value) & d$.value > 0, ]
      proj <- rep(NA_real_, nrow(d))

      if (nrow(fit_data) >= 2) {
        fit <- stats::lm(log(.value) ~ .t, data = fit_data)
        idx <- which(d$.date >= start_date)
        if (length(idx)) {
          # column is ".t" with no leading space
          proj[idx] <- exp(stats::predict(fit, newdata = d[idx, c(".t"), drop = FALSE]))
        }
      }

      dplyr::tibble(.row_id = d$.row_id, projection = proj)
    }) |>
    dplyr::ungroup() |>
    dplyr::arrange(.row_id)

  out$projection
}

# Deterministic two-group test panel. Group "a" and "b" are exact
# exponentials in calendar days, so a log-linear fit on any window must
# recover them to machine precision; val_noisy adds seeded lognormal noise.
make_test_panel <- function() {
  dates <- seq(as.Date("2014-01-01"), by = "month", length.out = 96)
  df <- dplyr::bind_rows(
    dplyr::tibble(grp = "a", date = dates, val = 100 * exp(0.002 * as.numeric(dates))),
    dplyr::tibble(grp = "b", date = dates, val = 50 * exp(0.001 * as.numeric(dates)))
  )
  set.seed(123)
  df$val_noisy <- df$val * exp(rnorm(nrow(df), 0, 0.005))
  df
}

test_that("logLinearProjection recovers an exact exponential trend", {
  df <- make_test_panel()
  res <- dplyr::mutate(
    df,
    proj = logLinearProjection(date, val, "2015-01-01", "2019-12-01", group = grp)
  )

  # NA before the calibration start: 12 months of 2014 per group
  expect_equal(sum(is.na(res$proj)), 24)
  expect_true(all(is.na(res$proj[res$date < as.Date("2015-01-01")])))

  # Exact recovery on and after start_date, including extrapolation
  # beyond end_date (relative tolerance; levels reach ~1e16)
  keep <- !is.na(res$proj)
  expect_equal(res$proj[keep], res$val[keep], tolerance = 1e-8)
})

test_that("logLinearProjection matches values pinned from version 0.2.0", {
  # Regression pins: computed 2026-06-10 from the deterministic panel
  # above, before the cur_data_all() -> pick() migration. Group "b" only.
  df <- make_test_panel()
  res <- dplyr::mutate(
    df,
    proj  = logLinearProjection(date, val, "2015-01-01", "2019-12-01", group = grp),
    projn = logLinearProjection(date, val_noisy, "2015-01-01", "2019-12-01", group = grp)
  )
  pin_dates <- as.Date(c("2015-01-01", "2020-06-01", "2021-12-01"))
  pins <- res[res$grp == "b" & res$date %in% pin_dates, ]
  expect_equal(pins$proj,
    c(6.8712240355e+08, 4.9667076130e+09, 8.5913610429e+09),
    tolerance = 1e-9
  )
  expect_equal(pins$projn,
    c(6.8625796507e+08, 4.9697634743e+09, 8.6011112959e+09),
    tolerance = 1e-9
  )
})

test_that("standalone data= argument matches the mutate() path", {
  df <- make_test_panel()
  in_mutate <- dplyr::mutate(
    df,
    proj = logLinearProjection(date, val, "2015-01-01", "2019-12-01", group = grp)
  )$proj
  standalone <- logLinearProjection(
    date, val, "2015-01-01", "2019-12-01",
    group = grp, data = df
  )
  expect_identical(standalone, in_mutate)
})

test_that("a grouped mutate works without an explicit group argument", {
  df <- make_test_panel()
  explicit <- dplyr::mutate(
    df,
    proj = logLinearProjection(date, val, "2015-01-01", "2019-12-01", group = grp)
  )$proj
  grouped <- df |>
    dplyr::group_by(grp) |>
    dplyr::mutate(proj = logLinearProjection(date, val, "2015-01-01", "2019-12-01")) |>
    dplyr::ungroup()
  expect_equal(grouped$proj, explicit)
})

test_that("no deprecation warning fires inside mutate()", {
  df <- make_test_panel()
  expect_no_warning(
    dplyr::mutate(
      df,
      proj = logLinearProjection(date, val, "2015-01-01", "2019-12-01", group = grp)
    )
  )
})

test_that("the pre-0.2.0 string interface fails with an informative error", {
  df <- make_test_panel()
  expect_error(
    logLinearProjection(df, "date", "val", "2015-01-01", "2019-12-01"),
    "bare column names"
  )
})

#' Build the Dallas Fed trimmed-mean PCE component panel
#'
#' Returns a long tibble with the raw inputs to the Federal Reserve Bank of
#' Dallas's trimmed-mean PCE inflation rate: monthly Fisher price index,
#' nominal expenditure, real quantity, monthly price change, the Fisher
#' (t-1, t) expenditure-share weight, and a flag indicating whether the
#' component was trimmed in that month and on which tail. Users can
#' replicate the trimmed-mean rate by collapsing this tibble to kept
#' (non-trimmed) components each month and taking the weight-renormalized
#' weighted mean of price changes.
#'
#' @details
#' Weights are Fisher-style: an unweighted average of the expenditure share
#' evaluated at base prices `P[t-1]` with quantities `Q[t-1]` and `Q[t]`,
#' renormalized to sum to 1 within each month. Real quantity is computed as
#' `nominal / price` from BEA NIPA Tables 2.4.5U and 2.4.4U respectively
#' (equivalent to Table 2.4.6U per the Dallas Fed's MATLAB note, but
#' available across the full sample without chained-dollar gaps).
#'
#' Trim assignment is the simple rank-based version: components are sorted
#' within each month by `price_change`, cumulative weight is accumulated,
#' and components whose running cumulative weight is below `alpha` are
#' flagged `"lower"`, while components whose cumulative weight before
#' adding their own contribution is at or above `1 - beta` are flagged
#' `"upper"`. Boundary components that straddle either threshold are
#' kept (treated as interior). The Dallas Fed's exact fractional-boundary
#' handling enters the rate calculation itself, not this panel-builder;
#' the resulting headline rate matches the Dallas Fed series to within a
#' few basis points.
#'
#' Months without full cross-sectional coverage (i.e., any component
#' missing this month or last) have `weight`, `is_trimmed`, and
#' `trim_side` set to `NA`.
#'
#' @param frequency Character. Frequency code passed to
#'   \code{\link{getNIPAFiles}}. Defaults to \code{"M"} (monthly).
#'   Currently the trimmed-mean panel is only meaningful at monthly
#'   frequency.
#' @param NIPA_data Optional pre-loaded NIPA tibble from
#'   \code{getNIPAFiles()}. If \code{NULL}, the function downloads it.
#' @param alpha Numeric in \[0, 1\]. Lower-tail trim share. Default
#'   \code{0.24}, the Dallas Fed published value.
#' @param beta Numeric in \[0, 1\]. Upper-tail trim share. Default
#'   \code{0.31}.
#' @param components Optional override for the component dictionary. Must
#'   be a tibble with columns \code{dallas_idx}, \code{name},
#'   \code{series_code}, \code{line_no}. Defaults to the packaged
#'   \code{\link{dallasTrimPCEcomponents}} (177 components).
#'
#' @return A \code{tbl_df} with one row per (date, component) and columns:
#' \describe{
#'   \item{date}{Month observation date.}
#'   \item{dallas_idx}{Component ordinal in the Dallas tech notes (1..178, 94 omitted).}
#'   \item{name}{Dallas Fed component name.}
#'   \item{series_code}{BEA NIPA series code (Table 2.4.4U).}
#'   \item{line_no}{BEA NIPA Table 2.4.4U line number.}
#'   \item{price}{Fisher price index (Table 2.4.4U).}
#'   \item{nominal}{Current-dollar outlay (Table 2.4.5U).}
#'   \item{quantity}{Real quantity (\code{nominal / price}).}
#'   \item{price_change}{Period-over-period fractional change in \code{price}. \code{NA} for the first observation per component.}
#'   \item{weight}{Fisher (t-1, t) expenditure-share weight, renormalized to sum to 1 within each full-coverage month. \code{NA} otherwise.}
#'   \item{is_trimmed}{Logical. \code{TRUE} if the component is in either tail this month and so dropped from the trimmed mean. \code{NA} when the month lacks full coverage.}
#'   \item{trim_side}{Character. \code{"lower"} or \code{"upper"} when trimmed; \code{NA} when kept (interior) or coverage incomplete.}
#' }
#'
#' @seealso \code{\link{dallasTrimPCEcomponents}}, \code{\link{getNIPAFiles}}
#'
#' @references
#' Dolmas, J. (2005). "Trimmed Mean PCE Inflation." Federal Reserve Bank
#' of Dallas Working Paper 0506.
#'
#' Dolmas, J. (2009, updated 2022-12-23). "PCE Inflation: Technical
#' Note." Federal Reserve Bank of Dallas.
#'
#' Atkinson, T., Dolmas, J., & Zarutskie, R. (2026). "Skewness warrants
#' caution as Trimmed Mean PCE inflation eases." Federal Reserve Bank of
#' Dallas, April 16, 2026.
#'
#' @importFrom dplyr arrange case_when coalesce filter group_by if_else
#'   inner_join lag mutate n pull select summarize transmute ungroup
#' @importFrom rlang .data
#' @importFrom utils data
#'
#' @examples
#' \dontrun{
#'   # Default 24/31 Dallas Fed trim
#'   panel <- getDallasTrimPCE()
#'
#'   # Replicate the monthly trimmed-mean rate (kept components, renormalized
#'   # to sum to 1):
#'   library(dplyr)
#'   tm_rate <- panel |>
#'     dplyr::filter(!is_trimmed) |>
#'     dplyr::group_by(date) |>
#'     dplyr::summarize(
#'       rate = sum(price_change * weight) / sum(weight),
#'       .groups = "drop"
#'     )
#'
#'   # What got trimmed in the latest month, by tail and weight:
#'   panel |>
#'     dplyr::filter(date == max(date), is_trimmed) |>
#'     dplyr::arrange(trim_side, dplyr::desc(weight)) |>
#'     dplyr::select(name, trim_side, weight, price_change)
#' }
#'
#' @export
getDallasTrimPCE <- function(frequency  = "M",
                             NIPA_data  = NULL,
                             alpha      = 0.24,
                             beta       = 0.31,
                             components = NULL) {

  stopifnot(is.numeric(alpha), is.numeric(beta),
            length(alpha) == 1, length(beta) == 1,
            alpha >= 0, beta >= 0, alpha + beta < 1)

  if (is.null(NIPA_data)) {
    NIPA_data <- getNIPAFiles(type = frequency)
  }
  if (is.null(components)) {
    e <- new.env()
    utils::data("dallasTrimPCEcomponents", package = "tidyusmacro", envir = e)
    components <- e$dallasTrimPCEcomponents
  }

  required <- c("dallas_idx", "name", "series_code", "line_no")
  missing  <- setdiff(required, names(components))
  if (length(missing)) {
    stop("`components` is missing required columns: ",
         paste(missing, collapse = ", "))
  }

  # Pull P (U20404) and N (U20405) restricted to the component line numbers,
  # rename to lower-case to match the rest of the panel.
  prices <- NIPA_data |>
    dplyr::filter(.data$TableId == "U20404",
                  .data$LineNo %in% components$line_no) |>
    dplyr::transmute(line_no = .data$LineNo,
                     date    = .data$date,
                     price   = .data$Value)

  noms <- NIPA_data |>
    dplyr::filter(.data$TableId == "U20405",
                  .data$LineNo %in% components$line_no) |>
    dplyr::transmute(line_no = .data$LineNo,
                     date    = .data$date,
                     nominal = .data$Value)

  panel <- prices |>
    dplyr::inner_join(noms, by = c("line_no", "date")) |>
    dplyr::inner_join(components, by = "line_no") |>
    dplyr::mutate(quantity = .data$nominal / .data$price)

  # Surface any component that never appears in the current NIPA pull.
  observed <- sort(unique(panel$line_no))
  expected <- sort(unique(components$line_no))
  if (!identical(observed, expected)) {
    miss <- setdiff(expected, observed)
    warning("Component line numbers missing from NIPA U20404/U20405: ",
            paste(miss, collapse = ", "))
  }

  # Per-component lags (within dallas_idx, ordered by date).
  panel <- panel |>
    dplyr::arrange(.data$dallas_idx, .data$date) |>
    dplyr::group_by(.data$dallas_idx) |>
    dplyr::mutate(
      price_change = .data$price / dplyr::lag(.data$price) - 1,
      P_lag        = dplyr::lag(.data$price),
      Q_lag        = dplyr::lag(.data$quantity)
    ) |>
    dplyr::ungroup()

  # Identify months with full cross-sectional coverage (every component has
  # both this month's and last month's observation). The trimmed-mean rate
  # is defined only on full-coverage months.
  ncomp <- nrow(components)
  full_dates <- panel |>
    dplyr::group_by(.data$date) |>
    dplyr::summarize(
      n_full = sum(!is.na(.data$price_change) &
                   !is.na(.data$P_lag) &
                   !is.na(.data$Q_lag)),
      .groups = "drop"
    ) |>
    dplyr::filter(.data$n_full == ncomp) |>
    dplyr::pull(.data$date)

  # Fisher (t-1, t) weights, renormalized to 1 within each full month.
  panel <- panel |>
    dplyr::mutate(
      .num1 = dplyr::if_else(.data$date %in% full_dates,
                             .data$quantity * .data$P_lag, NA_real_),
      .num2 = dplyr::if_else(.data$date %in% full_dates,
                             .data$Q_lag    * .data$P_lag, NA_real_)
    ) |>
    dplyr::group_by(.data$date) |>
    dplyr::mutate(
      .sum1  = sum(.data$.num1, na.rm = FALSE),
      .sum2  = sum(.data$.num2, na.rm = FALSE),
      weight = 0.5 * .data$.num1 / .data$.sum1 +
               0.5 * .data$.num2 / .data$.sum2
    ) |>
    dplyr::mutate(
      weight = .data$weight / sum(.data$weight, na.rm = FALSE)
    ) |>
    dplyr::ungroup()

  # Trim flags. Sort each month by price_change ascending; cumulative weight
  # determines which components fall in either tail.
  #   - lower : cum_w[i] < alpha           (entirely below the lower cutoff)
  #   - upper : cum_w[i-1] >= 1 - beta     (cumulative weight before this
  #                                         component already past 1-beta)
  panel <- panel |>
    dplyr::group_by(.data$date) |>
    dplyr::arrange(.data$price_change, .by_group = TRUE) |>
    dplyr::mutate(
      .cum_w  = cumsum(dplyr::coalesce(.data$weight, 0)),
      .prev_w = dplyr::lag(.data$.cum_w, default = 0),
      trim_side = dplyr::case_when(
        is.na(.data$weight)             ~ NA_character_,
        .data$.cum_w   <  alpha         ~ "lower",
        .data$.prev_w  >= 1 - beta      ~ "upper",
        TRUE                             ~ NA_character_
      ),
      is_trimmed = dplyr::if_else(is.na(.data$weight),
                                  NA, !is.na(.data$trim_side))
    ) |>
    dplyr::ungroup() |>
    dplyr::arrange(.data$date, .data$dallas_idx) |>
    dplyr::select(.data$date, .data$dallas_idx, .data$name,
                  .data$series_code, .data$line_no,
                  .data$price, .data$nominal, .data$quantity,
                  .data$price_change, .data$weight,
                  .data$is_trimmed, .data$trim_side)

  panel
}

#' Date breaks anchored to last data month (for ggplot)
#'
#' @description Create a breaks function for \code{scale_x_date()} that
#'   always includes the last actual data month and then selects every
#'   \code{n}th month counting backward.
#'
#' @param n Integer; keep every n-th month counting backward from \code{last}. Default 6.
#' @param last Date; the last (max) date in your data. Required to ensure no break
#'   is placed after your actual data.
#' @param decreasing Logical; if TRUE, return breaks in descending order. Default FALSE.
#'
#' @return A function usable in \code{scale_x_date(breaks = ...)}.
#' @export
#'
#' @examples
#' # Minimal reproducible example (avoid using the name `df`, which masks stats::df)
#' set.seed(1)
#' dat <- data.frame(
#'   date  = seq(as.Date("2023-01-01"), by = "month", length.out = 24),
#'   value = cumsum(rnorm(24))
#' )
#'
#' library(ggplot2)
#'
#' ggplot(dat, aes(date, value)) +
#'   geom_line() +
#'   scale_x_date(
#'     date_labels = "%b\n%Y",
#'     breaks = date_breaks_gg(n = 6, last = max(dat$date))
#'   ) +
#'   labs(x = NULL, y = NULL)
date_breaks_gg <- function(n = 6, last, decreasing = FALSE) {
  if (missing(last) || is.null(last)) {
    stop("date_breaks_gg(): `last` (the last data date) is required.", call. = FALSE)
  }
  if (n <= 0L) stop("date_breaks_gg(): `n` must be a positive integer.", call. = FALSE)

  # snap to month start without extra deps
  month_start <- function(x) as.Date(cut(as.Date(x), "month"))

  last <- month_start(last) # ensure we align to the month
  force(n); force(last); force(decreasing)

  function(limits) {
    # Build monthly candidates from the left limit up to *last* (never beyond)
    start_month <- month_start(limits[1])
    if (last < start_month) start_month <- last

    months <- seq.Date(from = start_month, to = last, by = "month")
    if (length(months) == 0L) return(months)

    k <- length(months)
    # Select indices counting backward from the last element: k, k-n, k-2n, ...
    idx <- seq.int(from = k, to = 1L, by = -n)
    out <- months[sort(idx)]  # ascending by default

    if (decreasing) rev(out) else out
  }
}

#' Load and Process Personal Consumption Expenditures (PCE) Inflation Data
#'
#' Downloads and processes BEA NIPA data to compute Personal Consumption Expenditures
#' (PCE) price indices with weights and growth measures. This is the Federal Reserve's
#' preferred inflation measure.
#'
#' The function performs the following steps:
#' \enumerate{
#'   \item Loads NIPA data using \code{\link{getNIPAFiles}} (or uses pre-loaded data).
#'   \item Extracts total PCE from table \code{"U20405"} (series code \code{"DPCERC"}).
#'   \item Computes PCE component weights as the nominal consumption share
#'         (component value divided by total PCE).
#'   \item Extracts quantity indices from table \code{"U20403"}.
#'   \item Loads price indices from table \code{"U20404"}, joins weights and quantities,
#'         and calculates period-over-period growth measures.
#' }
#'
#' @param frequency Character string indicating the frequency of the data.
#'                  Defaults to \code{"M"} (monthly).
#' @param NIPA_data Optional data frame. If provided, it will be used as the raw NIPA dataset
#'                  instead of loading fresh data with \code{getNIPAFiles()}.'
#'
#' @return A \code{tbl_df} (data frame) containing the PCE data with calculated variables.
#'
#' @importFrom dplyr select distinct lag mutate left_join group_by filter ungroup
#' @importFrom rlang .data
#' @examples
#' \dontrun{
#'   # Load monthly PCE data
#'   pce_data <- getPCEInflation("M")
#' }
#'
#' @export
getPCEInflation <- function(frequency = "M", NIPA_data = NULL) {

    # Load the full dataset using the specified frequency.
  if(is.null(NIPA_data)){
    full_data <- getNIPAFiles(type = frequency)
  } else{
    full_data <- NIPA_data
  }

  # ---------------------------------------------------------------------------
  # Step 1: Extract Total GDP data
  # ---------------------------------------------------------------------------
  # Filter for table "U20405" with SeriesCode "DPCERC" to get total GDP,
  # then rename the value column to TotalGDP.
  total_gdp <- full_data %>%
    filter(TableId == "U20405", SeriesCode == "DPCERC") %>%
    dplyr::select(date, TotalGDP = Value)

  # ---------------------------------------------------------------------------
  # Step 2: Calculate PCE weights
  # ---------------------------------------------------------------------------
  # For each date, join with the total GDP data to compute the PCE weight as:
  # weight = (value from U20405) / TotalGDP.
  # This approximates the nominal consumption share.
  pce_weight <- full_data %>%
    filter(TableId == "U20405") %>%
    left_join(total_gdp, by = "date") %>%
    mutate(PCEweight = Value / TotalGDP) %>%
    select(date, SeriesLabel, PCEweight) %>%
    distinct(date, SeriesLabel, .keep_all = TRUE)

  # ---------------------------------------------------------------------------
  # Step 3: Extract quantity data
  # ---------------------------------------------------------------------------
  # From table "U20403", extract the quantity data and rename the value column.
  PCE_Q <- full_data %>%
    filter(TableId == "U20403") %>%
    select(SeriesLabel, date, quantity = Value) %>%
    distinct(date, SeriesLabel, .keep_all = TRUE)

  # ---------------------------------------------------------------------------
  # Step 4: Process PCE Data
  # ---------------------------------------------------------------------------
  # From table "U20404", load the PCE data, join it with the computed PCE weights
  # and quantity data, then compute various growth measures.
  pce_df <- full_data %>%
    filter(TableId == "U20404") %>%
    left_join(pce_weight, by = c("date", "SeriesLabel")) %>%
    left_join(PCE_Q, by = c("date", "SeriesLabel")) %>%
    group_by(SeriesLabel) %>%
    # Calculate period-over-period changes:
    # DataValue_P1: 1-period (lag 1) percentage change
    mutate(DataValue_P1 = (Value - lag(Value, 1)) / lag(Value, 1)) %>%
    # DataValue_P3: 3-period change
    mutate(DataValue_P3 = Value / lag(Value, 3) - 1) %>%
    # DataValue_P6: 6-period change
    mutate(DataValue_P6 = Value / lag(Value, 6) - 1) %>%
    # Compute weighted 1-period change using the lagged weight.
    mutate(WDataValue_P1 = DataValue_P1 * lag(PCEweight, 1)) %>%
    # Annualize the weighted 1-period change.
    mutate(WDataValue_P1a = (1 + WDataValue_P1)^4 - 1) %>%
    ungroup()

  # Return the processed PCE data frame.
  return(pce_df)
}

#' Dallas Fed Trimmed-Mean PCE component dictionary
#'
#' A tibble mapping each of the 177 components used in the Federal Reserve
#' Bank of Dallas's trimmed-mean PCE inflation rate to its corresponding
#' series in BEA NIPA Table 2.4.4U (Fisher price index for personal
#' consumption expenditures by type of product, monthly).
#'
#' Components are organized as durables (1-41), nondurables (42-91),
#' services (92-177), and one NPISH aggregate (178). The 2009 tech notes
#' (Dolmas, "Trimmed Mean PCE Inflation," updated 2022-12-23) list 178
#' components; BEA combined two of them - Tenant-Occupied Stationary
#' Homes and Tenant Landlord Durables - into a single line of Table
#' 2.4.4U (`IA000629`) when the disaggregated series stopped in December
#' 2001. The mapping reflects that combination, yielding 177 rows;
#' `dallas_idx` runs 1..178 with 94 omitted (93 holds the merged item).
#'
#' @format A tibble with 177 rows and 5 variables:
#' \describe{
#'   \item{dallas_idx}{Integer. Ordinal position in the Dallas Fed tech-notes list.}
#'   \item{name}{Character. Component name as published by the Dallas Fed.}
#'   \item{series_code}{Character. BEA NIPA series code in Table 2.4.4U.}
#'   \item{line_no}{Integer. Line number in BEA Table 2.4.4U.}
#'   \item{bea_label}{Character. Label BEA publishes alongside `series_code`.}
#' }
#'
#' @source
#' Dolmas, J. (2009, updated 2022-12-23). "PCE Inflation: Technical
#' Note." Federal Reserve Bank of Dallas. BEA NIPA Table 2.4.4U.
#'
#' @references
#' Atkinson, T., Dolmas, J., & Zarutskie, R. (2026). "Skewness warrants
#' caution as Trimmed Mean PCE inflation eases." Federal Reserve Bank of
#' Dallas, April 16, 2026.
#'
#' @examples
#' data(dallasTrimPCEcomponents)
#' head(dallasTrimPCEcomponents)
#'
"dallasTrimPCEcomponents"

#' Download and Process BEA NIPA Files with Fast Row Expansion
#'
#' This function downloads and processes National Income and Product Accounts (NIPA)
#' data files from the BEA website. It reads the necessary register files, formats the
#' date column, and then uses the fast stringi functions together with tidyr's unnest()
#' to split the combined `TableId:LineNo` field into separate rows and columns. Finally,
#' it merges the datasets.
#'
#' @param location The URL or path where the BEA files are located.
#'   Default: "https://apps.bea.gov/national/Release/TXT/".
#' @param type A character string indicating the type of data to load.
#'   For example, "Q" for quarterly or "M" for monthly data. Default is "Q".
#'
#' @return A data frame containing the merged and formatted NIPA data.
#' @examples
#' \dontrun{
#'   nipadata <- getNIPAFiles(type = "Q")
#' }
#'
#' @export
getNIPAFiles <- function(location = "https://apps.bea.gov/national/Release/TXT/",
                         type = "Q") {

  # Helper function to format BEA period strings as dates.
  # For a quarterly identifier (e.g., "Q"), it converts the quarter into a month.
  BEA_format_date <- function(x) {
    year <- substr(x, 1, 4)
    identifier <- substr(x[1], 5, 5)

    if (identifier == "Q") {
      quarter <- substr(x, 6, 6)
      month <- ifelse(quarter == "1", "03",
                      ifelse(quarter == "2", "06",
                             ifelse(quarter == "3", "09", "12")))
    } else if (identifier == "M") {
      month <- substr(x, 6, 7)
    } else {
      stop("Unknown period identifier: ", identifier)
    }

    as.Date(paste(year, month, "01", sep = "-"), format = "%Y-%m-%d")
  }

  message("Start time: ", Sys.time())

  # Build the full URLs for the required files.
  series_url <- file.path(location, "SeriesRegister.txt")
  tables_url <- file.path(location, "TablesRegister.txt")
  data_url   <- file.path(location, paste0("nipadata", type, ".txt"))

  # Read the series and tables register files.
  series <- readr::read_csv(series_url, show_col_types = FALSE) %>%
    dplyr::rename(SeriesCode = `%SeriesCode`)

  tables <- readr::read_csv(tables_url, show_col_types = FALSE)

  message("Loading ", type, " data from ", data_url)
  data <- readr::read_csv(data_url, show_col_types = FALSE) %>%
    dplyr::rename(SeriesCode = `%SeriesCode`)

  # Format the date column.
  message("Formatting date column...")
  data <- data %>%
    dplyr::mutate(date = BEA_format_date(Period))

  # Process the TableId:LineNo column.
  # Instead of using separate_rows(), we use stringi to split the column and unnest the results.
  message("Splitting TableId:LineNo using stringi and unnesting...")
  final_data <- data %>%
    dplyr::left_join(series, by = "SeriesCode") %>%
    # Create a list-column by splitting on the pipe ("|")
    dplyr::mutate(TableIdList = stringi::stri_split_fixed(`TableId:LineNo`, pattern = "|", omit_empty = TRUE)) %>%
    tidyr::unnest(cols = c(TableIdList)) %>%
    # Extract the TableId (portion before ":") and LineNo (portion after ":")
    dplyr::mutate(
      TableId = stringi::stri_extract_first_regex(TableIdList, "^[^:]+"),
      LineNo  = stringi::stri_extract_last_regex(TableIdList, "[^:]+")
    ) %>%
    dplyr::left_join(tables, by = "TableId") %>%
    mutate(LineNo = as.numeric(LineNo))

  message("End time: ", Sys.time())
  return(final_data)
}

#' @importFrom utils globalVariables
utils::globalVariables(c(
  "series_id", "value", "TableId", "SeriesCode", "Value", "TotalGDP",
  "SeriesLabel", "PCEweight", "DataValue_P1", "WDataValue_P1",
  "LineNo", "Period", "TableIdList", "TableId:LineNo", "%SeriesCode",
  "..date", "..value", ".data", ".date", ".row_id"
))

#' Download and Merge FRED Series
#'
#' A flexible wrapper that downloads one or more data series from the St. Louis
#' Fed (FRED) API, optionally computes one-period percentage changes, and merges
#' them into a tidy tibble keyed by `date`.
#'
#' You may supply the series in two ways:
#' \itemize{
#'   \item \strong{Natural “\code{...}” style}:
#'     \code{getFRED(unrate = "UNRATE", payroll = "PAYEMS")}.
#'     Named arguments give friendly column names; unnamed arguments keep the
#'     (lower-case) ticker as the column name.
#'   \item \strong{Legacy style}: pass a single (optionally named) character
#'     vector—e.g.\ \code{c(unrate = "UNRATE", payroll = "PAYEMS")}—and/or use
#'     the \code{rename_variables=} argument.  This remains supported for
#'     backward compatibility.
#' }
#'
#' If you provide names in \code{...} \emph{and} a non-\code{NULL}
#' \code{rename_variables} vector, the function stops and prompts you to choose
#' a single naming method.
#'
#' @param ... One or more FRED series IDs.  Each element may be either
#'   \describe{
#'     \item{Unnamed character string}{The raw FRED ticker; column keeps the
#'       lowercase ticker name, e.g.\ \code{"UNRATE"}.}
#'     \item{Named character string}{The value is the FRED ticker and the name
#'       becomes the column label, e.g.\ \code{payroll = "PAYEMS"}.}
#'   }
#'   You may also pass a single character vector (named or unnamed) for
#'   compatibility with older code.
#' @param keep_all Logical.  \code{TRUE} (default) performs a full join that
#'   keeps all dates across series; \code{FALSE} performs an inner join.
#' @param rename_variables Optional character vector of new column names (one
#'   per series), retained for backward compatibility.  Supply \emph{either}
#'   this argument \emph{or} names in \code{...}, not both.
#' @param lagged Logical scalar or logical vector.  If \code{TRUE} (or the
#'   corresponding element is \code{TRUE}), the series is replaced by its
#'   one-period percentage change \eqn{(x_t / x_{t-1}) - 1}.  Recycled to match
#'   the number of series if length 1.
#'
#' @return A tibble with a \code{date} column and one column per requested
#'   series.
#' @examples
#' \dontrun{
#' # New interface
#' getFRED(unrate = "UNRATE", payroll = "PAYEMS")
#'
#' # Multiple unnamed series (columns become 'unrate' and 'payems')
#' getFRED("UNRATE", "PAYEMS")
#'
#' }
#' @importFrom readr read_csv cols
#' @importFrom dplyr full_join inner_join as_tibble
#' @importFrom purrr pmap reduce compact
#' @importFrom utils head
#' @importFrom httr GET status_code content user_agent
#' @export
getFRED <- function(...,
                    keep_all = TRUE,
                    rename_variables = NULL,
                    lagged = NULL) {
  ## ---------------------------------------------------------------
  ## 1.  Parse the dots --------------------------------------------
  ## ---------------------------------------------------------------
  dots <- list(...)
  if (length(dots) == 0) {
    stop("Provide at least one FRED series ID via the `...` arguments.")
  }

  flat <- unlist(dots, use.names = TRUE) # keeps slot names
  variables_raw <- unname(flat) # the series IDs
  n <- length(variables_raw)

  names_in_dots <- names(flat) # could be "" or NULL
  if (is.null(names_in_dots) || length(names_in_dots) == 0) {
    names_in_dots <- rep(NA_character_, n)
  }
  names_in_dots[names_in_dots == ""] <- NA_character_

  ## ---------------------------------------------------------------
  ## 2.  Resolve column names --------------------------------------
  ## ---------------------------------------------------------------
  if (!is.null(rename_variables)) {
    if (length(rename_variables) != n) {
      stop("`rename_variables` must have the same length as the series list.")
    }

    # Detect conflicting sources of names
    if (any(!is.na(names_in_dots))) {
      stop(
        "You supplied names via both the `...` arguments *and* ",
        "`rename_variables=`.  Please choose only one way to name columns."
      )
    }

    rename_vars <- rename_variables
  } else {
    rename_vars <- names_in_dots
  }

  ## ---------------------------------------------------------------
  ## 3.  Validate / recycle `lagged` -------------------------------
  ## ---------------------------------------------------------------
  if (is.null(lagged)) {
    lagged <- rep(FALSE, n)
  } else if (length(lagged) == 1 && n > 1) {
    lagged <- rep(lagged, n)
  } else if (length(lagged) != n) {
    stop("`lagged` must be length 1 or the same length as the series list.")
  }

  variables <- toupper(variables_raw)

  ## ---------------------------------------------------------------
  ## 4.  Helper: fetch one series ----------------------------------
  ## ---------------------------------------------------------------
  get_one <- function(var, col_name, do_lag) {
    url <- paste0(
      "https://fred.stlouisfed.org/graph/fredgraph.csv?id=", var
    )
    message("Downloading ", var)

    # Fetch via httr::GET because readr::read_csv() fails against FRED's
    # HTTP/2 endpoint ("HTTP/2 stream was not closed cleanly").
    df <- tryCatch(
      {
        resp <- httr::GET(
          url,
          httr::user_agent("tidyusmacro (https://github.com/mtkonczal/tidyusmacro)")
        )
        if (httr::status_code(resp) != 200) {
          stop("HTTP ", httr::status_code(resp))
        }
        body <- httr::content(resp, as = "text", encoding = "UTF-8")
        readr::read_csv(I(body), col_types = readr::cols())
      },
      error = function(e) {
        warning("Error downloading ", var, ": ", e$message)
        return(NULL)
      }
    )
    if (is.null(df) || ncol(df) < 2) {
      return(NULL)
    }

    names(df)[1:2] <- c("date", tolower(var))
    if (!inherits(df$date, "Date")) df$date <- as.Date(df$date)

    if (do_lag) {
      x <- df[[2]]
      df[[2]] <- c(NA, x[-1] / head(x, -1) - 1)
    }

    if (!is.na(col_name) && nzchar(col_name)) {
      names(df)[2] <- col_name
    }

    df
  }

  ## ---------------------------------------------------------------
  ## 5.  Download & merge ------------------------------------------
  ## ---------------------------------------------------------------
  dfs <- purrr::pmap(
    list(
      var = variables,
      col_name = rename_vars,
      do_lag = lagged
    ),
    get_one
  ) |>
    purrr::compact()

  if (length(dfs) == 0) {
    stop("No data downloaded successfully.")
  }

  joiner <- if (keep_all) {
    function(x, y) dplyr::full_join(x, y, by = "date")
  } else {
    function(x, y) dplyr::inner_join(x, y, by = "date")
  }

  purrr::reduce(dfs, joiner) |>
    dplyr::as_tibble()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getBLSFiles.R
\name{getBLSFiles}
\alias{getBLSFiles}
\title{Download and Process Bureau of Labor Statistics Data}
\usage{
getBLSFiles(data_source, email)
}
\arguments{
\item{data_source}{Character string specifying the BLS data source. Available options:
\describe{
\item{\code{"cpi"}}{Consumer Price Index - current data}
\item{\code{"eci"}}{Employment Cost Index (quarterly)}
\item{\code{"cex"}}{Consumer Expenditure Survey}
\item{\code{"jolts"}}{Job Openings and Labor Turnover Survey}
\item{\code{"cps"}}{Current Population Survey}
\item{\code{"ces"}}{Current Employment Statistics - all series}
\item{\code{"ces_allemp"}}{Current Employment Statistics - all employees, seasonally adjusted}
\item{\code{"ces_total"}}{Current Employment Statistics - total nonfarm employment}
\item{\code{"averageprice"}}{Average price data - current}
\item{\code{"food"}}{Average price data - food items}
\item{\code{"se"}}{State and metro area employment}
\item{\code{"su"}}{State and local area unemployment}
}}

\item{email}{Character string with your email address. Required by BLS for
identifying API users. Set as the HTTP User-Agent header.}
}
\value{
A tibble containing the merged data with columns for:
\item{series_id}{Unique identifier for each data series}
\item{date}{Observation date}
\item{value}{Numeric data value}
\item{...}{Additional metadata columns vary by data source (e.g., item codes,
industry codes, area codes)}
}
\description{
Downloads and processes data from Bureau of Labor Statistics (BLS) flat files.
Supports multiple data sources including CPI, ECI, JOLTS, CPS, CES, and others.
The function retrieves the main data file along with associated metadata files,
merges them, and returns a tidy tibble ready for analysis.
}
\details{
The function constructs URLs to BLS flat files at
\url{https://download.bls.gov/pub/time.series/}, downloads the series
metadata and auxiliary lookup tables, then downloads and merges the main
data file. Date parsing handles both monthly (most sources) and quarterly
(ECI) data frequencies.
}
\examples{
\dontrun{
  # Download CPI data
  cpi_data <- getBLSFiles("cpi", "your.email@example.com")

  # Download JOLTS data
  jolts_data <- getBLSFiles("jolts", "your.email@example.com")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDallasTrimPCE.R
\name{getDallasTrimPCE}
\alias{getDallasTrimPCE}
\title{Build the Dallas Fed trimmed-mean PCE component panel}
\usage{
getDallasTrimPCE(
  frequency = "M",
  NIPA_data = NULL,
  alpha = 0.24,
  beta = 0.31,
  components = NULL
)
}
\arguments{
\item{frequency}{Character. Frequency code passed to
\code{\link{getNIPAFiles}}. Defaults to \code{"M"} (monthly).
Currently the trimmed-mean panel is only meaningful at monthly
frequency.}

\item{NIPA_data}{Optional pre-loaded NIPA tibble from
\code{getNIPAFiles()}. If \code{NULL}, the function downloads it.}

\item{alpha}{Numeric in [0, 1]. Lower-tail trim share. Default
\code{0.24}, the Dallas Fed published value.}

\item{beta}{Numeric in [0, 1]. Upper-tail trim share. Default
\code{0.31}.}

\item{components}{Optional override for the component dictionary. Must
be a tibble with columns \code{dallas_idx}, \code{name},
\code{series_code}, \code{line_no}. Defaults to the packaged
\code{\link{dallasTrimPCEcomponents}} (177 components).}
}
\value{
A \code{tbl_df} with one row per (date, component) and columns:
\describe{
\item{date}{Month observation date.}
\item{dallas_idx}{Component ordinal in the Dallas tech notes (1..178, 94 omitted).}
\item{name}{Dallas Fed component name.}
\item{series_code}{BEA NIPA series code (Table 2.4.4U).}
\item{line_no}{BEA NIPA Table 2.4.4U line number.}
\item{price}{Fisher price index (Table 2.4.4U).}
\item{nominal}{Current-dollar outlay (Table 2.4.5U).}
\item{quantity}{Real quantity (\code{nominal / price}).}
\item{price_change}{Period-over-period fractional change in \code{price}. \code{NA} for the first observation per component.}
\item{weight}{Fisher (t-1, t) expenditure-share weight, renormalized to sum to 1 within each full-coverage month. \code{NA} otherwise.}
\item{is_trimmed}{Logical. \code{TRUE} if the component is in either tail this month and so dropped from the trimmed mean. \code{NA} when the month lacks full coverage.}
\item{trim_side}{Character. \code{"lower"} or \code{"upper"} when trimmed; \code{NA} when kept (interior) or coverage incomplete.}
}
}
\description{
Returns a long tibble with the raw inputs to the Federal Reserve Bank of
Dallas's trimmed-mean PCE inflation rate: monthly Fisher price index,
nominal expenditure, real quantity, monthly price change, the Fisher
(t-1, t) expenditure-share weight, and a flag indicating whether the
component was trimmed in that month and on which tail. Users can
replicate the trimmed-mean rate by collapsing this tibble to kept
(non-trimmed) components each month and taking the weight-renormalized
weighted mean of price changes.
}
\details{
Weights are Fisher-style: an unweighted average of the expenditure share
evaluated at base prices \code{P[t-1]} with quantities \code{Q[t-1]} and \code{Q[t]},
renormalized to sum to 1 within each month. Real quantity is computed as
\code{nominal / price} from BEA NIPA Tables 2.4.5U and 2.4.4U respectively
(equivalent to Table 2.4.6U per the Dallas Fed's MATLAB note, but
available across the full sample without chained-dollar gaps).

Trim assignment is the simple rank-based version: components are sorted
within each month by \code{price_change}, cumulative weight is accumulated,
and components whose running cumulative weight is below \code{alpha} are
flagged \code{"lower"}, while components whose cumulative weight before
adding their own contribution is at or above \code{1 - beta} are flagged
\code{"upper"}. Boundary components that straddle either threshold are
kept (treated as interior). The Dallas Fed's exact fractional-boundary
handling enters the rate calculation itself, not this panel-builder;
the resulting headline rate matches the Dallas Fed series to within a
few basis points.

Months without full cross-sectional coverage (i.e., any component
missing this month or last) have \code{weight}, \code{is_trimmed}, and
\code{trim_side} set to \code{NA}.
}
\examples{
\dontrun{
  # Default 24/31 Dallas Fed trim
  panel <- getDallasTrimPCE()

  # Replicate the monthly trimmed-mean rate (kept components, renormalized
  # to sum to 1):
  library(dplyr)
  tm_rate <- panel |>
    dplyr::filter(!is_trimmed) |>
    dplyr::group_by(date) |>
    dplyr::summarize(
      rate = sum(price_change * weight) / sum(weight),
      .groups = "drop"
    )

  # What got trimmed in the latest month, by tail and weight:
  panel |>
    dplyr::filter(date == max(date), is_trimmed) |>
    dplyr::arrange(trim_side, dplyr::desc(weight)) |>
    dplyr::select(name, trim_side, weight, price_change)
}

}
\references{
Dolmas, J. (2005). "Trimmed Mean PCE Inflation." Federal Reserve Bank
of Dallas Working Paper 0506.

Dolmas, J. (2009, updated 2022-12-23). "PCE Inflation: Technical
Note." Federal Reserve Bank of Dallas.

Atkinson, T., Dolmas, J., & Zarutskie, R. (2026). "Skewness warrants
caution as Trimmed Mean PCE inflation eases." Federal Reserve Bank of
Dallas, April 16, 2026.
}
\seealso{
\code{\link{dallasTrimPCEcomponents}}, \code{\link{getNIPAFiles}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-dallasTrimPCEcomponents.R
\docType{data}
\name{dallasTrimPCEcomponents}
\alias{dallasTrimPCEcomponents}
\title{Dallas Fed Trimmed-Mean PCE component dictionary}
\format{
A tibble with 177 rows and 5 variables:
\describe{
\item{dallas_idx}{Integer. Ordinal position in the Dallas Fed tech-notes list.}
\item{name}{Character. Component name as published by the Dallas Fed.}
\item{series_code}{Character. BEA NIPA series code in Table 2.4.4U.}
\item{line_no}{Integer. Line number in BEA Table 2.4.4U.}
\item{bea_label}{Character. Label BEA publishes alongside \code{series_code}.}
}
}
\source{
Dolmas, J. (2009, updated 2022-12-23). "PCE Inflation: Technical
Note." Federal Reserve Bank of Dallas. BEA NIPA Table 2.4.4U.
}
\usage{
dallasTrimPCEcomponents
}
\description{
A tibble mapping each of the 177 components used in the Federal Reserve
Bank of Dallas's trimmed-mean PCE inflation rate to its corresponding
series in BEA NIPA Table 2.4.4U (Fisher price index for personal
consumption expenditures by type of product, monthly).
}
\details{
Components are organized as durables (1-41), nondurables (42-91),
services (92-177), and one NPISH aggregate (178). The 2009 tech notes
(Dolmas, "Trimmed Mean PCE Inflation," updated 2022-12-23) list 178
components; BEA combined two of them - Tenant-Occupied Stationary
Homes and Tenant Landlord Durables - into a single line of Table
2.4.4U (\code{IA000629}) when the disaggregated series stopped in December
2001. The mapping reflects that combination, yielding 177 rows;
\code{dallas_idx} runs 1..178 with 94 omitted (93 holds the merged item).
}
\examples{
data(dallasTrimPCEcomponents)
head(dallasTrimPCEcomponents)

}
\references{
Atkinson, T., Dolmas, J., & Zarutskie, R. (2026). "Skewness warrants
caution as Trimmed Mean PCE inflation eases." Federal Reserve Bank of
Dallas, April 16, 2026.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_esp.R
\docType{data}
\name{esp_pal}
\alias{esp_pal}
\title{ESP Color Palette}
\format{
An object of class \code{character} of length 3.
}
\usage{
esp_pal
}
\description{
Named vector of ESP-branded colors.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFRED.R
\name{getFRED}
\alias{getFRED}
\title{Download and Merge FRED Series}
\usage{
getFRED(..., keep_all = TRUE, rename_variables = NULL, lagged = NULL)
}
\arguments{
\item{...}{One or more FRED series IDs.  Each element may be either
\describe{
\item{Unnamed character string}{The raw FRED ticker; column keeps the
lowercase ticker name, e.g.\ \code{"UNRATE"}.}
\item{Named character string}{The value is the FRED ticker and the name
becomes the column label, e.g.\ \code{payroll = "PAYEMS"}.}
}
You may also pass a single character vector (named or unnamed) for
compatibility with older code.}

\item{keep_all}{Logical.  \code{TRUE} (default) performs a full join that
keeps all dates across series; \code{FALSE} performs an inner join.}

\item{rename_variables}{Optional character vector of new column names (one
per series), retained for backward compatibility.  Supply \emph{either}
this argument \emph{or} names in \code{...}, not both.}

\item{lagged}{Logical scalar or logical vector.  If \code{TRUE} (or the
corresponding element is \code{TRUE}), the series is replaced by its
one-period percentage change \eqn{(x_t / x_{t-1}) - 1}.  Recycled to match
the number of series if length 1.}
}
\value{
A tibble with a \code{date} column and one column per requested
series.
}
\description{
A flexible wrapper that downloads one or more data series from the St. Louis
Fed (FRED) API, optionally computes one-period percentage changes, and merges
them into a tidy tibble keyed by \code{date}.
}
\details{
You may supply the series in two ways:
\itemize{
\item \strong{Natural “\code{...}” style}:
\code{getFRED(unrate = "UNRATE", payroll = "PAYEMS")}.
Named arguments give friendly column names; unnamed arguments keep the
(lower-case) ticker as the column name.
\item \strong{Legacy style}: pass a single (optionally named) character
vector—e.g.\ \code{c(unrate = "UNRATE", payroll = "PAYEMS")}—and/or use
the \code{rename_variables=} argument.  This remains supported for
backward compatibility.
}

If you provide names in \code{...} \emph{and} a non-\code{NULL}
\code{rename_variables} vector, the function stops and prompts you to choose
a single naming method.
}
\examples{
\dontrun{
# New interface
getFRED(unrate = "UNRATE", payroll = "PAYEMS")

# Multiple unnamed series (columns become 'unrate' and 'payems')
getFRED("UNRATE", "PAYEMS")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_esp.R
\docType{data}
\name{esp_navy}
\alias{esp_navy}
\title{ESP Primary Color (Navy)}
\format{
An object of class \code{character} of length 1.
}
\usage{
esp_navy
}
\description{
A standalone color value for quick use.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_breaks_n.R
\name{date_breaks_n}
\alias{date_breaks_n}
\title{Create evenly spaced breaks}
\usage{
date_breaks_n(dates, n = 6, decreasing = TRUE)
}
\arguments{
\item{dates}{A vector of dates.}

\item{n}{Integer, keep every n-th date (default = 6).}

\item{decreasing}{Logical, if TRUE (default) sorts dates in descending order.}
}
\value{
A vector of dates suitable for use as ggplot2 axis breaks.
}
\description{
Generate a sequence of date breaks for ggplot scales,
taking every \code{n}th unique date.
}
\examples{
library(ggplot2)
library(dplyr)

df <- tibble(
  date = seq.Date(as.Date("2020-01-01"), as.Date("2025-01-01"), by = "month"),
  value = rnorm(61)
)

ggplot(df, aes(date, value)) +
  geom_line() +
  scale_x_date(breaks = date_breaks_n(df$date, 6))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_breaks_gg.R
\name{date_breaks_gg}
\alias{date_breaks_gg}
\title{Date breaks anchored to last data month (for ggplot)}
\usage{
date_breaks_gg(n = 6, last, decreasing = FALSE)
}
\arguments{
\item{n}{Integer; keep every n-th month counting backward from \code{last}. Default 6.}

\item{last}{Date; the last (max) date in your data. Required to ensure no break
is placed after your actual data.}

\item{decreasing}{Logical; if TRUE, return breaks in descending order. Default FALSE.}
}
\value{
A function usable in \code{scale_x_date(breaks = ...)}.
}
\description{
Create a breaks function for \code{scale_x_date()} that
always includes the last actual data month and then selects every
\code{n}th month counting backward.
}
\examples{
# Minimal reproducible example (avoid using the name `df`, which masks stats::df)
set.seed(1)
dat <- data.frame(
  date  = seq(as.Date("2023-01-01"), by = "month", length.out = 24),
  value = cumsum(rnorm(24))
)

library(ggplot2)

ggplot(dat, aes(date, value)) +
  geom_line() +
  scale_x_date(
    date_labels = "\%b\n\%Y",
    breaks = date_breaks_gg(n = 6, last = max(dat$date))
  ) +
  labs(x = NULL, y = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_esp.R
\name{esp_theme}
\alias{esp_theme}
\alias{theme_esp}
\alias{scale_color_esp}
\alias{scale_fill_esp}
\alias{scale_colour_esp}
\title{ESP Theme and Color Scales}
\usage{
theme_esp(base_family = "Public Sans")

scale_color_esp(...)

scale_fill_esp(...)

scale_colour_esp(...)
}
\arguments{
\item{base_family}{Base font family for the theme. Defaults to "Public Sans".}

\item{...}{Passed to the underlying ggplot2 scale functions.}
}
\value{
A ggplot2 theme or scale object.
}
\description{
Custom theme and color palette for Economic Security Project graphics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPCEInflation.R
\name{getPCEInflation}
\alias{getPCEInflation}
\title{Load and Process Personal Consumption Expenditures (PCE) Inflation Data}
\usage{
getPCEInflation(frequency = "M", NIPA_data = NULL)
}
\arguments{
\item{frequency}{Character string indicating the frequency of the data.
Defaults to \code{"M"} (monthly).}

\item{NIPA_data}{Optional data frame. If provided, it will be used as the raw NIPA dataset
instead of loading fresh data with \code{getNIPAFiles()}.'}
}
\value{
A \code{tbl_df} (data frame) containing the PCE data with calculated variables.
}
\description{
Downloads and processes BEA NIPA data to compute Personal Consumption Expenditures
(PCE) price indices with weights and growth measures. This is the Federal Reserve's
preferred inflation measure.
}
\details{
The function performs the following steps:
\enumerate{
\item Loads NIPA data using \code{\link{getNIPAFiles}} (or uses pre-loaded data).
\item Extracts total PCE from table \code{"U20405"} (series code \code{"DPCERC"}).
\item Computes PCE component weights as the nominal consumption share
(component value divided by total PCE).
\item Extracts quantity indices from table \code{"U20403"}.
\item Loads price indices from table \code{"U20404"}, joins weights and quantities,
and calculates period-over-period growth measures.
}
}
\examples{
\dontrun{
  # Load monthly PCE data
  pce_data <- getPCEInflation("M")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getNIPAFiles.R
\name{getNIPAFiles}
\alias{getNIPAFiles}
\title{Download and Process BEA NIPA Files with Fast Row Expansion}
\usage{
getNIPAFiles(
  location = "https://apps.bea.gov/national/Release/TXT/",
  type = "Q"
)
}
\arguments{
\item{location}{The URL or path where the BEA files are located.
Default: "https://apps.bea.gov/national/Release/TXT/".}

\item{type}{A character string indicating the type of data to load.
For example, "Q" for quarterly or "M" for monthly data. Default is "Q".}
}
\value{
A data frame containing the merged and formatted NIPA data.
}
\description{
This function downloads and processes National Income and Product Accounts (NIPA)
data files from the BEA website. It reads the necessary register files, formats the
date column, and then uses the fast stringi functions together with tidyr's unnest()
to split the combined \code{TableId:LineNo} field into separate rows and columns. Finally,
it merges the datasets.
}
\examples{
\dontrun{
  nipadata <- getNIPAFiles(type = "Q")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getUnrateFRED.R
\name{getUnrateFRED}
\alias{getUnrateFRED}
\title{Get Full Unemployment Rate from FRED}
\usage{
getUnrateFRED()
}
\value{
A tibble with columns:
\item{date}{Observation date}
\item{unemploy_level}{Civilian unemployment level (in thousands)}
\item{lf_level}{Civilian labor force level (in thousands)}
\item{full_unrate}{Unemployment rate (decimal)}
}
\description{
Downloads the civilian unemployment level and labor force level from
FRED, and calculates the unemployment rate as
\eqn{\text{unemploy\_level} / \text{lf\_level}}.
}
\examples{
\dontrun{
  getUnrateFRED()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-cesDiffusionIndex.R
\docType{data}
\name{cesDiffusionIndex}
\alias{cesDiffusionIndex}
\title{cesDiffusionIndex Dataset}
\format{
A tibble with 250 rows and 2 variables:
\describe{
\item{ces_industry_code}{A character vector containing the industry codes (e.g., "10-11330000").}
\item{ces_industry_title}{A character vector containing the titles of the industries (e.g., "Logging").}
}
}
\source{
U.S. Bureau of Labor Statistics (BLS)
}
\usage{
cesDiffusionIndex
}
\description{
A tibble with 250 rows and 2 columns representing industry codes and corresponding industry titles.
}
\details{
This dataset contains information on different industries, where each row corresponds to an industry
defined by its unique code and a descriptive title. It is useful for analyses that require linking industry
classifications to descriptive labels.
}
\examples{
# Load the dataset
data(cesDiffusionIndex)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLinearProjection.R
\name{logLinearProjection}
\alias{logLinearProjection}
\title{Log-Linear Projection (data-masked, dplyr-native)}
\usage{
logLinearProjection(
  date,
  value,
  start_date,
  end_date,
  group = NULL,
  data = NULL
)
}
\arguments{
\item{date}{Bare column name for the date variable (coercible to Date).}

\item{value}{Bare column name for the positive numeric series to project.}

\item{start_date}{Date or string coercible to Date; start of calibration.}

\item{end_date}{Date or string coercible to Date; end of calibration.}

\item{group}{Optional bare column name to group by before projecting.}

\item{data}{Optional data frame. If omitted, uses the columns visible in
the current data mask (e.g., inside \code{mutate()}) via
\code{dplyr::pick()}.}
}
\value{
A numeric vector \code{projection} aligned to the input rows; \code{NA}
before \code{start_date}. Respects grouping if \code{group} is supplied.
}
\description{
Fits a log-linear trend \code{log(value) ~ t} on a calibration window and
projects it for rows on/after \code{start_date}. Designed for use inside
dplyr verbs (no need to pass \code{.}).
}
